from ..broker import Broker


class DeviceWirelessBroker(Broker):
    controller = "device_wirelesses"

    def index(self, **kwargs):
        """Lists the available device wirelesses. Any of the inputs listed may be be used to narrow the list; other inputs will be ignored. Of the various ways to query lists, using this method is most efficient.

            **Inputs**

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceID: The internal NetMRI identifier of each device from which wireless setting was collected.
             :type DeviceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceID: The internal NetMRI identifier of each device from which wireless setting was collected.
             :type DeviceID: Array of Integer

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessID: The internal NetMRI identifier of the device wireless.
             :type DeviceWirelessID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessID: The internal NetMRI identifier of the device wireless.
             :type DeviceWirelessID: Array of Integer

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param InterfaceID: The internal NetMRI identifier of a local interface for the device wireless.
             :type InterfaceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param InterfaceID: The internal NetMRI identifier of a local interface for the device wireless.
             :type InterfaceID: Array of Integer

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param StationID: The internal NetMRI identifier of a station.
             :type StationID: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param StationID: The internal NetMRI identifier of a station.
             :type StationID: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceGroupID: The internal NetMRI identifier of the device groups to which to limit the results.
             :type DeviceGroupID: Array of Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param timestamp: The data returned will represent the device wirelesses as of this date and time. If omitted, the result will indicate the most recently collected data.
             :type timestamp: DateTime

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param methods: A list of device wireless methods. The listed methods will be called on each device wireless returned and included in the output. Available methods are: device, interface.
             :type methods: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param include: A list of associated object types to include in the output. The listed associations will be returned as outputs named according to the association name (see outputs below). Available includes are: device, interface.
             :type include: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` DeviceWirelessID

             :param sort: The data field(s) to use for sorting the output. Default is DeviceWirelessID. Valid values are DeviceWirelessID, DeviceWirelessStartTime, DeviceWirelessEndTime, DeviceWirelessChangedCols, DeviceWirelessTimestamp, DataSourceID, DeviceID, ifIndex, InterfaceID, StationID, DesiredSSID, StationRole, WEPEnabledInd, WEPAllowedInd, WEPOnlyTrafficInd, WEPICVErrorCount, WEPDefaultKeyLen1, WEPDefaultKeyLen2, WEPDefaultKeyLen3, WEPDefaultKeyLen4.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each DeviceWireless. Valid values are DeviceWirelessID, DeviceWirelessStartTime, DeviceWirelessEndTime, DeviceWirelessChangedCols, DeviceWirelessTimestamp, DataSourceID, DeviceID, ifIndex, InterfaceID, StationID, DesiredSSID, StationRole, WEPEnabledInd, WEPAllowedInd, WEPOnlyTrafficInd, WEPICVErrorCount, WEPDefaultKeyLen1, WEPDefaultKeyLen2, WEPDefaultKeyLen3, WEPDefaultKeyLen4. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return device_wirelesses: An array of the DeviceWireless objects that match the specified input criteria.
             :rtype device_wirelesses: Array of DeviceWireless

            """

        return self.api_list_request(self._get_method_fullname("index"), kwargs)

    def show(self, **kwargs):
        """Shows the details for the specified device wireless.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceWirelessID: The internal NetMRI identifier of the device wireless.
             :type DeviceWirelessID: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param methods: A list of device wireless methods. The listed methods will be called on each device wireless returned and included in the output. Available methods are: device, interface.
             :type methods: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param include: A list of associated object types to include in the output. The listed associations will be returned as outputs named according to the association name (see outputs below). Available includes are: device, interface.
             :type include: Array of String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return device_wireless: The device wireless identified by the specified DeviceWirelessID.
             :rtype device_wireless: DeviceWireless

            """

        return self.api_request(self._get_method_fullname("show"), kwargs)

    def search(self, **kwargs):
        """Lists the available device wirelesses matching the input criteria. This method provides a more flexible search interface than the index method, but searching using this method is more demanding on the system and will not perform to the same level as the index method. The input fields listed below will be used as in the index method, to filter the result, along with the optional query string and XML filter described below.

            **Inputs**

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DataSourceID: The internal NetMRI identifier for the collector NetMRI that collected this data record.
             :type DataSourceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DataSourceID: The internal NetMRI identifier for the collector NetMRI that collected this data record.
             :type DataSourceID: Array of Integer

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DesiredSSID: The service set identifier(SSID) of desired wireless device.
             :type DesiredSSID: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DesiredSSID: The service set identifier(SSID) of desired wireless device.
             :type DesiredSSID: Array of String

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceID: The internal NetMRI identifier of each device from which wireless setting was collected.
             :type DeviceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceID: The internal NetMRI identifier of each device from which wireless setting was collected.
             :type DeviceID: Array of Integer

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessChangedCols: The fields that changed between this revision of the record and the previous revision.
             :type DeviceWirelessChangedCols: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessChangedCols: The fields that changed between this revision of the record and the previous revision.
             :type DeviceWirelessChangedCols: Array of String

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessEndTime: The ending effective time of this record, or empty if still in effect.
             :type DeviceWirelessEndTime: DateTime

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessEndTime: The ending effective time of this record, or empty if still in effect.
             :type DeviceWirelessEndTime: Array of DateTime

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessID: The internal NetMRI identifier of the device wireless.
             :type DeviceWirelessID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessID: The internal NetMRI identifier of the device wireless.
             :type DeviceWirelessID: Array of Integer

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessStartTime: The starting effective time of this record.
             :type DeviceWirelessStartTime: DateTime

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessStartTime: The starting effective time of this record.
             :type DeviceWirelessStartTime: Array of DateTime

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessTimestamp: The date and time this record was collected or calculated.
             :type DeviceWirelessTimestamp: DateTime

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceWirelessTimestamp: The date and time this record was collected or calculated.
             :type DeviceWirelessTimestamp: Array of DateTime

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param InterfaceID: The internal NetMRI identifier of a local interface for the device wireless.
             :type InterfaceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param InterfaceID: The internal NetMRI identifier of a local interface for the device wireless.
             :type InterfaceID: Array of Integer

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param StationID: The internal NetMRI identifier of a station.
             :type StationID: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param StationID: The internal NetMRI identifier of a station.
             :type StationID: Array of String

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param StationRole: The role name of the station.
             :type StationRole: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param StationRole: The role name of the station.
             :type StationRole: Array of String

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param WEPAllowedInd: A flag indicates whether a wired equivalent privacy is allowed or not?
             :type WEPAllowedInd: Boolean

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param WEPAllowedInd: A flag indicates whether a wired equivalent privacy is allowed or not?
             :type WEPAllowedInd: Array of Boolean

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param WEPDefaultKeyLen1: The initial default key length of wired equivalent privacy.
             :type WEPDefaultKeyLen1: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param WEPDefaultKeyLen1: The initial default key length of wired equivalent privacy.
             :type WEPDefaultKeyLen1: Array of String

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param WEPDefaultKeyLen2: The second default key length of wired equivalent privacy.
             :type WEPDefaultKeyLen2: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param WEPDefaultKeyLen2: The second default key length of wired equivalent privacy.
             :type WEPDefaultKeyLen2: Array of String

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param WEPDefaultKeyLen3: The third default key length of wired equivalent privacy.
             :type WEPDefaultKeyLen3: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param WEPDefaultKeyLen3: The third default key length of wired equivalent privacy.
             :type WEPDefaultKeyLen3: Array of String

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param WEPDefaultKeyLen4: The fourth default key length of wired equivalent privacy.
             :type WEPDefaultKeyLen4: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param WEPDefaultKeyLen4: The fourth default key length of wired equivalent privacy.
             :type WEPDefaultKeyLen4: Array of String

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param WEPEnabledInd: A flag indicates whether a wired equivalent privacy is enabled or not?
             :type WEPEnabledInd: Boolean

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param WEPEnabledInd: A flag indicates whether a wired equivalent privacy is enabled or not?
             :type WEPEnabledInd: Array of Boolean

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param WEPICVErrorCount: The total number of integrity check errors in WEP.
             :type WEPICVErrorCount: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param WEPICVErrorCount: The total number of integrity check errors in WEP.
             :type WEPICVErrorCount: Array of Integer

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param WEPOnlyTrafficInd: A flag indicates whether a wired equivalent privacy is traffic or not?
             :type WEPOnlyTrafficInd: Boolean

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param WEPOnlyTrafficInd: A flag indicates whether a wired equivalent privacy is traffic or not?
             :type WEPOnlyTrafficInd: Array of Boolean

            |  ``api version min:`` 2.4
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param ifIndex: The SNMP index of a local interface for the device wireless table entry.
             :type ifIndex: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param ifIndex: The SNMP index of a local interface for the device wireless table entry.
             :type ifIndex: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceGroupID: The internal NetMRI identifier of the device groups to which to limit the results.
             :type DeviceGroupID: Array of Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param timestamp: The data returned will represent the device wirelesses as of this date and time. If omitted, the result will indicate the most recently collected data.
             :type timestamp: DateTime

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param methods: A list of device wireless methods. The listed methods will be called on each device wireless returned and included in the output. Available methods are: device, interface.
             :type methods: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param include: A list of associated object types to include in the output. The listed associations will be returned as outputs named according to the association name (see outputs below). Available includes are: device, interface.
             :type include: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` DeviceWirelessID

             :param sort: The data field(s) to use for sorting the output. Default is DeviceWirelessID. Valid values are DeviceWirelessID, DeviceWirelessStartTime, DeviceWirelessEndTime, DeviceWirelessChangedCols, DeviceWirelessTimestamp, DataSourceID, DeviceID, ifIndex, InterfaceID, StationID, DesiredSSID, StationRole, WEPEnabledInd, WEPAllowedInd, WEPOnlyTrafficInd, WEPICVErrorCount, WEPDefaultKeyLen1, WEPDefaultKeyLen2, WEPDefaultKeyLen3, WEPDefaultKeyLen4.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each DeviceWireless. Valid values are DeviceWirelessID, DeviceWirelessStartTime, DeviceWirelessEndTime, DeviceWirelessChangedCols, DeviceWirelessTimestamp, DataSourceID, DeviceID, ifIndex, InterfaceID, StationID, DesiredSSID, StationRole, WEPEnabledInd, WEPAllowedInd, WEPOnlyTrafficInd, WEPICVErrorCount, WEPDefaultKeyLen1, WEPDefaultKeyLen2, WEPDefaultKeyLen3, WEPDefaultKeyLen4. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param query: This value will be matched against device wirelesses, looking to see if one or more of the listed attributes contain the passed value. You may also surround the value with '/' and '/' to perform a regular expression search rather than a containment operation. Any record that matches will be returned. The attributes searched are: DataSourceID, DesiredSSID, DeviceID, DeviceWirelessChangedCols, DeviceWirelessEndTime, DeviceWirelessID, DeviceWirelessStartTime, DeviceWirelessTimestamp, InterfaceID, StationID, StationRole, WEPAllowedInd, WEPDefaultKeyLen1, WEPDefaultKeyLen2, WEPDefaultKeyLen3, WEPDefaultKeyLen4, WEPEnabledInd, WEPICVErrorCount, WEPOnlyTrafficInd, ifIndex.
             :type query: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param xml_filter: A SetFilter XML structure to further refine the search. The SetFilter will be applied AFTER any search query or field values, but before any limit options. The limit and pagination will be enforced after the filter. Remind that this kind of filter may be costly and inefficient if not associated with a database filtering.
             :type xml_filter: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return device_wirelesses: An array of the DeviceWireless objects that match the specified input criteria.
             :rtype device_wirelesses: Array of DeviceWireless

            """

        return self.api_list_request(self._get_method_fullname("search"), kwargs)

    def find(self, **kwargs):
        """Lists the available device wirelesses matching the input specification. This provides the most flexible search specification of all the query mechanisms, enabling searching using comparison operations other than equality. However, it is more complex to use and will not perform as efficiently as the index or search methods. In the input descriptions below, 'field names' refers to the following fields: DataSourceID, DesiredSSID, DeviceID, DeviceWirelessChangedCols, DeviceWirelessEndTime, DeviceWirelessID, DeviceWirelessStartTime, DeviceWirelessTimestamp, InterfaceID, StationID, StationRole, WEPAllowedInd, WEPDefaultKeyLen1, WEPDefaultKeyLen2, WEPDefaultKeyLen3, WEPDefaultKeyLen4, WEPEnabledInd, WEPICVErrorCount, WEPOnlyTrafficInd, ifIndex.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DataSourceID: The operator to apply to the field DataSourceID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DataSourceID: The internal NetMRI identifier for the collector NetMRI that collected this data record. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DataSourceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DataSourceID: If op_DataSourceID is specified, the field named in this input will be compared to the value in DataSourceID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DataSourceID must be specified if op_DataSourceID is specified.
             :type val_f_DataSourceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DataSourceID: If op_DataSourceID is specified, this value will be compared to the value in DataSourceID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DataSourceID must be specified if op_DataSourceID is specified.
             :type val_c_DataSourceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DesiredSSID: The operator to apply to the field DesiredSSID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DesiredSSID: The service set identifier(SSID) of desired wireless device. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DesiredSSID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DesiredSSID: If op_DesiredSSID is specified, the field named in this input will be compared to the value in DesiredSSID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DesiredSSID must be specified if op_DesiredSSID is specified.
             :type val_f_DesiredSSID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DesiredSSID: If op_DesiredSSID is specified, this value will be compared to the value in DesiredSSID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DesiredSSID must be specified if op_DesiredSSID is specified.
             :type val_c_DesiredSSID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DeviceID: The operator to apply to the field DeviceID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DeviceID: The internal NetMRI identifier of each device from which wireless setting was collected. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DeviceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DeviceID: If op_DeviceID is specified, the field named in this input will be compared to the value in DeviceID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DeviceID must be specified if op_DeviceID is specified.
             :type val_f_DeviceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DeviceID: If op_DeviceID is specified, this value will be compared to the value in DeviceID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DeviceID must be specified if op_DeviceID is specified.
             :type val_c_DeviceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DeviceWirelessChangedCols: The operator to apply to the field DeviceWirelessChangedCols. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DeviceWirelessChangedCols: The fields that changed between this revision of the record and the previous revision. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DeviceWirelessChangedCols: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DeviceWirelessChangedCols: If op_DeviceWirelessChangedCols is specified, the field named in this input will be compared to the value in DeviceWirelessChangedCols using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DeviceWirelessChangedCols must be specified if op_DeviceWirelessChangedCols is specified.
             :type val_f_DeviceWirelessChangedCols: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DeviceWirelessChangedCols: If op_DeviceWirelessChangedCols is specified, this value will be compared to the value in DeviceWirelessChangedCols using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DeviceWirelessChangedCols must be specified if op_DeviceWirelessChangedCols is specified.
             :type val_c_DeviceWirelessChangedCols: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DeviceWirelessEndTime: The operator to apply to the field DeviceWirelessEndTime. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DeviceWirelessEndTime: The ending effective time of this record, or empty if still in effect. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DeviceWirelessEndTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DeviceWirelessEndTime: If op_DeviceWirelessEndTime is specified, the field named in this input will be compared to the value in DeviceWirelessEndTime using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DeviceWirelessEndTime must be specified if op_DeviceWirelessEndTime is specified.
             :type val_f_DeviceWirelessEndTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DeviceWirelessEndTime: If op_DeviceWirelessEndTime is specified, this value will be compared to the value in DeviceWirelessEndTime using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DeviceWirelessEndTime must be specified if op_DeviceWirelessEndTime is specified.
             :type val_c_DeviceWirelessEndTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DeviceWirelessID: The operator to apply to the field DeviceWirelessID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DeviceWirelessID: The internal NetMRI identifier of the device wireless. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DeviceWirelessID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DeviceWirelessID: If op_DeviceWirelessID is specified, the field named in this input will be compared to the value in DeviceWirelessID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DeviceWirelessID must be specified if op_DeviceWirelessID is specified.
             :type val_f_DeviceWirelessID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DeviceWirelessID: If op_DeviceWirelessID is specified, this value will be compared to the value in DeviceWirelessID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DeviceWirelessID must be specified if op_DeviceWirelessID is specified.
             :type val_c_DeviceWirelessID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DeviceWirelessStartTime: The operator to apply to the field DeviceWirelessStartTime. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DeviceWirelessStartTime: The starting effective time of this record. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DeviceWirelessStartTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DeviceWirelessStartTime: If op_DeviceWirelessStartTime is specified, the field named in this input will be compared to the value in DeviceWirelessStartTime using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DeviceWirelessStartTime must be specified if op_DeviceWirelessStartTime is specified.
             :type val_f_DeviceWirelessStartTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DeviceWirelessStartTime: If op_DeviceWirelessStartTime is specified, this value will be compared to the value in DeviceWirelessStartTime using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DeviceWirelessStartTime must be specified if op_DeviceWirelessStartTime is specified.
             :type val_c_DeviceWirelessStartTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DeviceWirelessTimestamp: The operator to apply to the field DeviceWirelessTimestamp. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DeviceWirelessTimestamp: The date and time this record was collected or calculated. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DeviceWirelessTimestamp: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DeviceWirelessTimestamp: If op_DeviceWirelessTimestamp is specified, the field named in this input will be compared to the value in DeviceWirelessTimestamp using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DeviceWirelessTimestamp must be specified if op_DeviceWirelessTimestamp is specified.
             :type val_f_DeviceWirelessTimestamp: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DeviceWirelessTimestamp: If op_DeviceWirelessTimestamp is specified, this value will be compared to the value in DeviceWirelessTimestamp using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DeviceWirelessTimestamp must be specified if op_DeviceWirelessTimestamp is specified.
             :type val_c_DeviceWirelessTimestamp: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_InterfaceID: The operator to apply to the field InterfaceID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. InterfaceID: The internal NetMRI identifier of a local interface for the device wireless. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_InterfaceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_InterfaceID: If op_InterfaceID is specified, the field named in this input will be compared to the value in InterfaceID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_InterfaceID must be specified if op_InterfaceID is specified.
             :type val_f_InterfaceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_InterfaceID: If op_InterfaceID is specified, this value will be compared to the value in InterfaceID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_InterfaceID must be specified if op_InterfaceID is specified.
             :type val_c_InterfaceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_StationID: The operator to apply to the field StationID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. StationID: The internal NetMRI identifier of a station. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_StationID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_StationID: If op_StationID is specified, the field named in this input will be compared to the value in StationID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_StationID must be specified if op_StationID is specified.
             :type val_f_StationID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_StationID: If op_StationID is specified, this value will be compared to the value in StationID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_StationID must be specified if op_StationID is specified.
             :type val_c_StationID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_StationRole: The operator to apply to the field StationRole. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. StationRole: The role name of the station. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_StationRole: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_StationRole: If op_StationRole is specified, the field named in this input will be compared to the value in StationRole using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_StationRole must be specified if op_StationRole is specified.
             :type val_f_StationRole: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_StationRole: If op_StationRole is specified, this value will be compared to the value in StationRole using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_StationRole must be specified if op_StationRole is specified.
             :type val_c_StationRole: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_WEPAllowedInd: The operator to apply to the field WEPAllowedInd. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. WEPAllowedInd: A flag indicates whether a wired equivalent privacy is allowed or not? For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_WEPAllowedInd: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_WEPAllowedInd: If op_WEPAllowedInd is specified, the field named in this input will be compared to the value in WEPAllowedInd using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_WEPAllowedInd must be specified if op_WEPAllowedInd is specified.
             :type val_f_WEPAllowedInd: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_WEPAllowedInd: If op_WEPAllowedInd is specified, this value will be compared to the value in WEPAllowedInd using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_WEPAllowedInd must be specified if op_WEPAllowedInd is specified.
             :type val_c_WEPAllowedInd: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_WEPDefaultKeyLen1: The operator to apply to the field WEPDefaultKeyLen1. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. WEPDefaultKeyLen1: The initial default key length of wired equivalent privacy. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_WEPDefaultKeyLen1: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_WEPDefaultKeyLen1: If op_WEPDefaultKeyLen1 is specified, the field named in this input will be compared to the value in WEPDefaultKeyLen1 using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_WEPDefaultKeyLen1 must be specified if op_WEPDefaultKeyLen1 is specified.
             :type val_f_WEPDefaultKeyLen1: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_WEPDefaultKeyLen1: If op_WEPDefaultKeyLen1 is specified, this value will be compared to the value in WEPDefaultKeyLen1 using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_WEPDefaultKeyLen1 must be specified if op_WEPDefaultKeyLen1 is specified.
             :type val_c_WEPDefaultKeyLen1: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_WEPDefaultKeyLen2: The operator to apply to the field WEPDefaultKeyLen2. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. WEPDefaultKeyLen2: The second default key length of wired equivalent privacy. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_WEPDefaultKeyLen2: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_WEPDefaultKeyLen2: If op_WEPDefaultKeyLen2 is specified, the field named in this input will be compared to the value in WEPDefaultKeyLen2 using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_WEPDefaultKeyLen2 must be specified if op_WEPDefaultKeyLen2 is specified.
             :type val_f_WEPDefaultKeyLen2: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_WEPDefaultKeyLen2: If op_WEPDefaultKeyLen2 is specified, this value will be compared to the value in WEPDefaultKeyLen2 using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_WEPDefaultKeyLen2 must be specified if op_WEPDefaultKeyLen2 is specified.
             :type val_c_WEPDefaultKeyLen2: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_WEPDefaultKeyLen3: The operator to apply to the field WEPDefaultKeyLen3. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. WEPDefaultKeyLen3: The third default key length of wired equivalent privacy. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_WEPDefaultKeyLen3: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_WEPDefaultKeyLen3: If op_WEPDefaultKeyLen3 is specified, the field named in this input will be compared to the value in WEPDefaultKeyLen3 using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_WEPDefaultKeyLen3 must be specified if op_WEPDefaultKeyLen3 is specified.
             :type val_f_WEPDefaultKeyLen3: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_WEPDefaultKeyLen3: If op_WEPDefaultKeyLen3 is specified, this value will be compared to the value in WEPDefaultKeyLen3 using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_WEPDefaultKeyLen3 must be specified if op_WEPDefaultKeyLen3 is specified.
             :type val_c_WEPDefaultKeyLen3: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_WEPDefaultKeyLen4: The operator to apply to the field WEPDefaultKeyLen4. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. WEPDefaultKeyLen4: The fourth default key length of wired equivalent privacy. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_WEPDefaultKeyLen4: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_WEPDefaultKeyLen4: If op_WEPDefaultKeyLen4 is specified, the field named in this input will be compared to the value in WEPDefaultKeyLen4 using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_WEPDefaultKeyLen4 must be specified if op_WEPDefaultKeyLen4 is specified.
             :type val_f_WEPDefaultKeyLen4: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_WEPDefaultKeyLen4: If op_WEPDefaultKeyLen4 is specified, this value will be compared to the value in WEPDefaultKeyLen4 using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_WEPDefaultKeyLen4 must be specified if op_WEPDefaultKeyLen4 is specified.
             :type val_c_WEPDefaultKeyLen4: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_WEPEnabledInd: The operator to apply to the field WEPEnabledInd. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. WEPEnabledInd: A flag indicates whether a wired equivalent privacy is enabled or not? For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_WEPEnabledInd: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_WEPEnabledInd: If op_WEPEnabledInd is specified, the field named in this input will be compared to the value in WEPEnabledInd using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_WEPEnabledInd must be specified if op_WEPEnabledInd is specified.
             :type val_f_WEPEnabledInd: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_WEPEnabledInd: If op_WEPEnabledInd is specified, this value will be compared to the value in WEPEnabledInd using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_WEPEnabledInd must be specified if op_WEPEnabledInd is specified.
             :type val_c_WEPEnabledInd: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_WEPICVErrorCount: The operator to apply to the field WEPICVErrorCount. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. WEPICVErrorCount: The total number of integrity check errors in WEP. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_WEPICVErrorCount: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_WEPICVErrorCount: If op_WEPICVErrorCount is specified, the field named in this input will be compared to the value in WEPICVErrorCount using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_WEPICVErrorCount must be specified if op_WEPICVErrorCount is specified.
             :type val_f_WEPICVErrorCount: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_WEPICVErrorCount: If op_WEPICVErrorCount is specified, this value will be compared to the value in WEPICVErrorCount using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_WEPICVErrorCount must be specified if op_WEPICVErrorCount is specified.
             :type val_c_WEPICVErrorCount: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_WEPOnlyTrafficInd: The operator to apply to the field WEPOnlyTrafficInd. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. WEPOnlyTrafficInd: A flag indicates whether a wired equivalent privacy is traffic or not? For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_WEPOnlyTrafficInd: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_WEPOnlyTrafficInd: If op_WEPOnlyTrafficInd is specified, the field named in this input will be compared to the value in WEPOnlyTrafficInd using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_WEPOnlyTrafficInd must be specified if op_WEPOnlyTrafficInd is specified.
             :type val_f_WEPOnlyTrafficInd: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_WEPOnlyTrafficInd: If op_WEPOnlyTrafficInd is specified, this value will be compared to the value in WEPOnlyTrafficInd using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_WEPOnlyTrafficInd must be specified if op_WEPOnlyTrafficInd is specified.
             :type val_c_WEPOnlyTrafficInd: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_ifIndex: The operator to apply to the field ifIndex. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. ifIndex: The SNMP index of a local interface for the device wireless table entry. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_ifIndex: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_ifIndex: If op_ifIndex is specified, the field named in this input will be compared to the value in ifIndex using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_ifIndex must be specified if op_ifIndex is specified.
             :type val_f_ifIndex: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_ifIndex: If op_ifIndex is specified, this value will be compared to the value in ifIndex using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_ifIndex must be specified if op_ifIndex is specified.
             :type val_c_ifIndex: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceGroupID: The internal NetMRI identifier of the device groups to which to limit the results.
             :type DeviceGroupID: Array of Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param timestamp: The data returned will represent the device wirelesses as of this date and time. If omitted, the result will indicate the most recently collected data.
             :type timestamp: DateTime

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param methods: A list of device wireless methods. The listed methods will be called on each device wireless returned and included in the output. Available methods are: device, interface.
             :type methods: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param include: A list of associated object types to include in the output. The listed associations will be returned as outputs named according to the association name (see outputs below). Available includes are: device, interface.
             :type include: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` DeviceWirelessID

             :param sort: The data field(s) to use for sorting the output. Default is DeviceWirelessID. Valid values are DeviceWirelessID, DeviceWirelessStartTime, DeviceWirelessEndTime, DeviceWirelessChangedCols, DeviceWirelessTimestamp, DataSourceID, DeviceID, ifIndex, InterfaceID, StationID, DesiredSSID, StationRole, WEPEnabledInd, WEPAllowedInd, WEPOnlyTrafficInd, WEPICVErrorCount, WEPDefaultKeyLen1, WEPDefaultKeyLen2, WEPDefaultKeyLen3, WEPDefaultKeyLen4.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each DeviceWireless. Valid values are DeviceWirelessID, DeviceWirelessStartTime, DeviceWirelessEndTime, DeviceWirelessChangedCols, DeviceWirelessTimestamp, DataSourceID, DeviceID, ifIndex, InterfaceID, StationID, DesiredSSID, StationRole, WEPEnabledInd, WEPAllowedInd, WEPOnlyTrafficInd, WEPICVErrorCount, WEPDefaultKeyLen1, WEPDefaultKeyLen2, WEPDefaultKeyLen3, WEPDefaultKeyLen4. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param xml_filter: A SetFilter XML structure to further refine the search. The SetFilter will be applied AFTER any search query or field values, but before any limit options. The limit and pagination will be enforced after the filter. Remind that this kind of filter may be costly and inefficient if not associated with a database filtering.
             :type xml_filter: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return device_wirelesses: An array of the DeviceWireless objects that match the specified input criteria.
             :rtype device_wirelesses: Array of DeviceWireless

            """

        return self.api_list_request(self._get_method_fullname("find"), kwargs)

    def data_source(self, **kwargs):
        """The collector NetMRI that collected this data record.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceWirelessID: The internal NetMRI identifier of the device wireless.
             :type DeviceWirelessID: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return : The collector NetMRI that collected this data record.
             :rtype : DataSource

            """

        return self.api_request(self._get_method_fullname("data_source"), kwargs)

    def device(self, **kwargs):
        """The device from which this data was collected.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceWirelessID: The internal NetMRI identifier of the device wireless.
             :type DeviceWirelessID: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return : The device from which this data was collected.
             :rtype : Device

            """

        return self.api_request(self._get_method_fullname("device"), kwargs)

    def interface(self, **kwargs):
        """interface

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceWirelessID: The internal NetMRI identifier of the device wireless.
             :type DeviceWirelessID: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return : interface
             :rtype : Interface

            """

        return self.api_request(self._get_method_fullname("interface"), kwargs)

    def infradevice(self, **kwargs):
        """The device from which this data was collected.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceWirelessID: The internal NetMRI identifier of the device wireless.
             :type DeviceWirelessID: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return : The device from which this data was collected.
             :rtype : InfraDevice

            """

        return self.api_request(self._get_method_fullname("infradevice"), kwargs)



from ..remote import RemoteModel


class ConfigMgmtJobManagementLibraryGridRemote(RemoteModel):
    """



    |  ``scriptid:`` none
    |  ``attribute type:`` string

    |  ``name:`` none
    |  ``attribute type:`` string

    |  ``language:`` none
    |  ``attribute type:`` string

    |  ``category:`` none
    |  ``attribute type:`` string

    |  ``description:`` none
    |  ``attribute type:`` string

    |  ``created_by:`` none
    |  ``attribute type:`` string

    |  ``created_at:`` none
    |  ``attribute type:`` string

    |  ``updated_by:`` none
    |  ``attribute type:`` string

    |  ``updated_at:`` none
    |  ``attribute type:`` string

    """

    properties = ("scriptid",
                  "name",
                  "language",
                  "category",
                  "description",
                  "created_by",
                  "created_at",
                  "updated_by",
                  "updated_at",
                  )



from ..remote import RemoteModel


class FailOverConfigurationRemote(RemoteModel):
    """



    |  ``id:`` none
    |  ``attribute type:`` string

    |  ``unit_id:`` none
    |  ``attribute type:`` string

    |  ``status:`` none
    |  ``attribute type:`` string

    |  ``virtual_ip:`` none
    |  ``attribute type:`` string

    |  ``virtual_hostname:`` none
    |  ``attribute type:`` string

    |  ``replication_direct_connect:`` none
    |  ``attribute type:`` string

    |  ``primary_index:`` none
    |  ``attribute type:`` string

    |  ``first_management_ip:`` none
    |  ``attribute type:`` string

    |  ``first_management_hostname:`` none
    |  ``attribute type:`` string

    |  ``second_management_ip:`` none
    |  ``attribute type:`` string

    |  ``second_management_hostname:`` none
    |  ``attribute type:`` string

    |  ``replication_port:`` none
    |  ``attribute type:`` string

    |  ``first_replication_ip:`` none
    |  ``attribute type:`` string

    |  ``first_replication_subnet:`` none
    |  ``attribute type:`` string

    |  ``second_replication_ip:`` none
    |  ``attribute type:`` string

    |  ``second_replication_subnet:`` none
    |  ``attribute type:`` string

    """

    properties = ("id",
                  "unit_id",
                  "status",
                  "virtual_ip",
                  "virtual_hostname",
                  "replication_direct_connect",
                  "primary_index",
                  "first_management_ip",
                  "first_management_hostname",
                  "second_management_ip",
                  "second_management_hostname",
                  "replication_port",
                  "first_replication_ip",
                  "first_replication_subnet",
                  "second_replication_ip",
                  "second_replication_subnet",
                  )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_infoblox_netmri
----------------------------------

Tests for `infoblox_netmri` module.
"""

import unittest
import json
from requests import Session

from httmock import with_httmock, urlmatch
from mock import patch

from infoblox_netmri.client import InfobloxNetMRI
from infoblox_netmri.easy import NetMRIEasy


class MockResponse:

    def __init__(self, status_code=200, content=b"{}", data=None, is_headers=False):
        self.status_code = status_code
        self.content = content
        self.headers = self._json_headers() if is_headers else self._empty_headers()
        self.data = {} if data is None else data

    @staticmethod
    def _empty_headers():
        return {'content-type': []}

    @staticmethod
    def _json_headers():
        return {'content-type': ['application/json']}

    def json(self):
        return json.dumps(self.data)

    def iter_content(self):
        if self.data:
            temp = []
            for symbol in json.dumps(self.data):
                temp.append(symbol.encode())
            return temp
        return []


@urlmatch(path=r"^/api/authenticate$")
def authenticate_response(_url, _request):
    return {'status_code': 200, 'content': b"{}", 'headers': {'content-type': []}}


@urlmatch(path=r"^/api/3.1/dis_sessions/open")
def open_dis_session(_url, _request):
    return {'status_code': 200,
            'headers': {'content-type': json.dumps(['application/json'])},
            'content': b'{"dis_session": {"JobID": 4, "SessionID": "149088011782626-7348", "RemoteUsername": "admin", '
                       b'"RemoteIPAddress": "1.2.3.4", "Timestamp": "1855-01-30 06:21:57", "UnitIDList": [0], '
                       b'"_class": "DisSession"}}'}


@urlmatch(path=r'/api/3.1/dis_sessions/close')
def close_session(_url, _request):
    return {'status_code': 200, 'content': b'{}', 'headers': {'content-type': []}}


@urlmatch(path=r"^/api/3.1/cli_connections/open")
def open_cli_session(_url, _request):
    return {'status_code': 200, 'content': b"{}", 'headers': {'content-type': []}}


@urlmatch(path=r"^/api/3.1/jobs/[0-9]+$")
def job_response(_url, _request):
    return {'status_code': 200, 'content': b'{"job": {}}', 'headers': {'content-type': []}}


@urlmatch(path=r"^/api/3.1/devices/index$")
def devices_list_response(_url, _request):
    return {'status_code': 200, 'content': b'{}', 'headers': {'content-type': []}}


device = {'DeviceType': 'unknown', 'InfraDeviceInd': False,
          'MgmtServerDeviceID': 0, 'DeviceStartTime': '2016-11-16 04:10:12',
          'DeviceMAC': '', 'DeviceAssurance': 0,
          'DeviceTimestamp': '2016-11-16 04:10:12', 'DeviceModel': None, 'DeviceSysDescr': None,
          'DeviceSysContact': None, 'DeviceID': 1, 'DeviceName': 'unknown',
          'DataSourceID': 0, 'NetworkDeviceInd': False, 'DeviceEndTime': None,
          'VirtualNetworkID': 1, 'DeviceIPDotted': '60.1.200.180', 'VirtualInd': False,
          'DeviceVersion': None, 'DeviceVendor': None, 'DeviceIPNumeric': 1006749876, 'DeviceOUI': None,
          'DeviceDNSName': None, 'DeviceChangedCols': None, 'DeviceSysName': None,
          'DeviceFirstOccurrenceTime': '2016-11-16 04:10:12', 'DeviceNetBIOSName': None,
          'DeviceSysLocation': None, 'DeviceUniqueKey': None, 'ParentDeviceID': 0, '_class':
              'Device', 'DeviceAddlInfo': None}


def devices_list(*_, **__):
    global device
    device2 = device.copy()
    device2.update({'DeviceID': 2})
    devices = [device, device2]

    return {'devices': devices}


def single_device(*_, **__):
    global device
    return {'device': device}


def send_command_result(*_, **__):
    return {'command_response': "OK"}


class TestInfobloxNetmri(unittest.TestCase):
    opts = {"host": "localhost",
            "username": "admin",
            "password": "admin",
            "api_version": "3.1"}

    def tearDown(self):
        pass

    @with_httmock(authenticate_response)
    def test_init_valid(self):
        netmri = InfobloxNetMRI(**self.opts)
        self.assertEqual(type(netmri), InfobloxNetMRI)

    @with_httmock(authenticate_response)
    def test_init_missing(self):
        self.assertRaises(TypeError, InfobloxNetMRI,
                          host="localhost",
                          user="admin")

        self.assertRaises(TypeError, InfobloxNetMRI,
                          host="localhost",
                          password="admin")

        self.assertRaises(TypeError, InfobloxNetMRI,
                          user="admin",
                          password="admin")

    @with_httmock(authenticate_response)
    def test_init_ssl_verify_bool(self):
        netmri = InfobloxNetMRI(**self.opts)
        self.assertEqual(False, netmri.ssl_verify)

        netmri = InfobloxNetMRI(ssl_verify="no", **self.opts)
        self.assertEqual(False, netmri.ssl_verify)

        netmri = InfobloxNetMRI(ssl_verify="off", **self.opts)
        self.assertEqual(False, netmri.ssl_verify)

        netmri = InfobloxNetMRI(ssl_verify=False, **self.opts)
        self.assertEqual(False, netmri.ssl_verify)

        netmri = InfobloxNetMRI(ssl_verify="false", **self.opts)
        self.assertEqual(False, netmri.ssl_verify)

        netmri = InfobloxNetMRI(ssl_verify="true", **self.opts)
        self.assertEqual(True, netmri.ssl_verify)

        netmri = InfobloxNetMRI(ssl_verify='yes', **self.opts)
        self.assertEqual(True, netmri.ssl_verify)

        netmri = InfobloxNetMRI(ssl_verify='on', **self.opts)
        self.assertEqual(True, netmri.ssl_verify)

        netmri = InfobloxNetMRI(ssl_verify=True, **self.opts)
        self.assertEqual(True, netmri.ssl_verify)

    @with_httmock(authenticate_response)
    @patch('infoblox_netmri.client.isfile')
    def test_init_ssl_verify_file(self, mock_isfile):
        mock_isfile.return_value = True
        netmri = InfobloxNetMRI(ssl_verify='/some/path.crt', **self.opts)
        self.assertEqual('/some/path.crt', netmri.ssl_verify)

        mock_isfile.return_value = False
        self.assertRaises(ValueError, InfobloxNetMRI, ssl_verify='/some/path.crt', **self.opts)

    @with_httmock(authenticate_response, job_response)
    def test_show(self):
        netmri = InfobloxNetMRI(**self.opts)
        with patch.object(Session, 'request', return_value=MockResponse()) as mock_request:
            netmri.show('job', 123)
            mock_request.assert_called_with("get",
                                            'https://localhost/api/3.1/jobs/123',
                                            headers={'Content-type': 'application/json'},
                                            data=None,
                                            stream=True)

    @with_httmock(authenticate_response, job_response)
    def test_show_with_string(self):
        netmri = InfobloxNetMRI(**self.opts)
        with patch.object(Session, 'request', return_value=MockResponse()) as mock_request:
            netmri.show('job', '232')
            mock_request.assert_called_with("get",
                                            'https://localhost/api/3.1/jobs/232',
                                            headers={'Content-type': 'application/json'},
                                            data=None,
                                            stream=True)

    @with_httmock(authenticate_response, job_response)
    def test_delete(self):
        netmri = InfobloxNetMRI(**self.opts)
        with patch.object(Session, 'request', return_value=MockResponse()) as mock_request:
            netmri.delete('job', 123)
            mock_request.assert_called_with("delete",
                                            'https://localhost/api/3.1/jobs/123',
                                            headers={'Content-type': 'application/json'},
                                            data=None,
                                            stream=True)

    @with_httmock(authenticate_response, job_response)
    def test_delete_with_string(self):
        netmri = InfobloxNetMRI(**self.opts)
        with patch.object(Session, 'request', return_value=MockResponse()) as mock_request:
            netmri.delete('job', '321')
            mock_request.assert_called_with("delete",
                                            'https://localhost/api/3.1/jobs/321',
                                            headers={'Content-type': 'application/json'},
                                            data=None,
                                            stream=True)

    @with_httmock(authenticate_response)
    def test_get_device_broker(self):
        netmri = InfobloxNetMRI(**self.opts)
        broker = netmri.get_broker('Device')
        self.assertEqual(broker.__class__.__name__, 'DeviceBroker', "Device broker import error")

        broker = netmri.get_broker('AccessChange')
        self.assertEqual(broker.__class__.__name__, 'AccessChangeBroker', "AccessChangeBroker broker import error")

    @with_httmock(authenticate_response, devices_list_response)
    def test_get_devices_list(self):
        netmri = InfobloxNetMRI(**self.opts)
        broker = netmri.get_broker('Device')
        with patch.object(Session, 'request', return_value=MockResponse()) as mock_request:
            broker.index()
            mock_request.assert_called_with("post",
                                            'https://localhost/api/3.1/devices/index',
                                            headers={'Content-type': 'application/json'},
                                            data='{}',
                                            stream=True)

    @with_httmock(authenticate_response, devices_list_response)
    def test_get_device(self):
        netmri = InfobloxNetMRI(**self.opts)
        broker = netmri.get_broker('Device')
        with patch.object(Session, 'request', return_value=MockResponse()) as mock_request:
            broker.show(DeviceID=2)
            mock_request.assert_called_with("post",
                                            'https://localhost/api/3.1/devices/show',
                                            headers={'Content-type': 'application/json'},
                                            data='{"DeviceID": 2}',
                                            stream=True)

    @with_httmock(authenticate_response)
    def test_package_creation(self):
        netmri = InfobloxNetMRI(**self.opts)
        package = "infoblox_netmri.api.broker.v3_1_0.device_broker.DeviceBroker"
        self.assertEqual(package, netmri._get_broker_package("Device"))

    @with_httmock(authenticate_response)
    def test_check_return_values(self):
        netmri = InfobloxNetMRI(**self.opts)
        broker = netmri.get_broker('Device')
        with patch.object(netmri.session, 'request', return_value=MockResponse(data=devices_list(), is_headers=True)):
            res = broker.index()
            self.assertEqual(res[0].DeviceID, 1, "Wrong id device 1")
            self.assertEqual(res[1].DeviceID, 2, "Wrong id device 2")

    @with_httmock(authenticate_response)
    def test_check_single_device(self):
        netmri = InfobloxNetMRI(**self.opts)
        broker = netmri.get_broker('Device')
        with patch.object(netmri.session, 'request', return_value=MockResponse(data=single_device(), is_headers=True)):
            res = broker.show(DeviceID=1)
            self.assertEqual(res.DeviceID, 1, "Wrong id")

    @with_httmock(authenticate_response, open_dis_session, open_cli_session, close_session)
    def test_easy(self):
        init_data = {'device_id': 1, 'job_id': 1, 'batch_id': 1}
        init_data.update(
            {'api_url': 'https://localhost',
             'http_username': 'admin',
             'http_password': 'admin',
             'api_version': '3.1'}
        )
        with NetMRIEasy(**init_data) as easy:
            self.assertEqual(easy.device_id, 1)
            self.assertEqual(easy.job_id, 1)
            self.assertEqual(easy.batch_id, 1)
            self.assertIsNotNone(easy.dis_session)
            self.assertIsNotNone(easy.cli_connection)

    @with_httmock(authenticate_response, open_dis_session, open_cli_session, close_session)
    def test_easy_send_command(self):
        init_data = {'device_id': 1, 'job_id': 1, 'batch_id': 1}
        init_data.update(
            {'api_url': 'http://localhost',
             'http_username': 'admin',
             'http_password': 'admin',
             'api_version': '3.1'}
        )
        with NetMRIEasy(**init_data) as easy:
            with patch.object(easy.client.session, 'request',
                              return_value=MockResponse(data=send_command_result(), is_headers=True)):
                res = easy.send_command("show info")
                self.assertEqual(res, "OK")

    @with_httmock(authenticate_response, open_dis_session, open_cli_session, close_session)
    def test_easy_empty_params(self):
        # empty constructor test
        self.assertRaises(RuntimeError, NetMRIEasy)


if __name__ == '__main__':
    import sys

    sys.exit(unittest.main())



from ..remote import RemoteModel


class VulnerabilityViewerVulnerabilitiesGridRemote(RemoteModel):
    """



    |  ``id:`` none
    |  ``attribute type:`` string

    |  ``Status:`` none
    |  ``attribute type:`` string

    |  ``OverrideNote:`` none
    |  ``attribute type:`` string

    |  ``Timestamp:`` none
    |  ``attribute type:`` string

    |  ``Network:`` none
    |  ``attribute type:`` string

    |  ``DeviceIPDotted:`` none
    |  ``attribute type:`` string

    |  ``DeviceIPNumeric:`` none
    |  ``attribute type:`` string

    |  ``DeviceID:`` none
    |  ``attribute type:`` string

    |  ``DeviceName:`` none
    |  ``attribute type:`` string

    |  ``DeviceVendor:`` none
    |  ``attribute type:`` string

    |  ``DeviceModel:`` none
    |  ``attribute type:`` string

    |  ``DeviceOS:`` none
    |  ``attribute type:`` string

    |  ``DeviceType:`` none
    |  ``attribute type:`` string

    |  ``StatusInd:`` none
    |  ``attribute type:`` string

    """

    properties = ("id",
                  "Status",
                  "OverrideNote",
                  "Timestamp",
                  "Network",
                  "DeviceIPDotted",
                  "DeviceIPNumeric",
                  "DeviceID",
                  "DeviceName",
                  "DeviceVendor",
                  "DeviceModel",
                  "DeviceOS",
                  "DeviceType",
                  "StatusInd",
                  )



from ..remote import RemoteModel


class VrfAwareDevicesGridRemote(RemoteModel):
    """
    list of VRF aware devices


    |  ``id:`` The internal system identifier for the worksheet.
    |  ``attribute type:`` number

    |  ``DeviceID:`` DeviceID.
    |  ``attribute type:`` number

    |  ``DeviceName:`` Device Name.
    |  ``attribute type:`` string

    |  ``DeviceIPDotted:`` Device Ip Address.
    |  ``attribute type:`` string

    |  ``DeviceIPNumeric:`` Device Ip Numeric Address.
    |  ``attribute type:`` string

    |  ``DeviceNetwork:`` Indicates the network view which the device is associated to.
    |  ``attribute type:`` string

    |  ``DeviceVirtualNetworkID:`` The internal identifier for the network which the device is associated to.
    |  ``attribute type:`` string

    |  ``ifIndex:`` Interface ID.
    |  ``attribute type:`` string

    |  ``ifName:`` Interface Name.
    |  ``attribute type:`` string

    |  ``ifNameSort:`` Interface Name Sort.
    |  ``attribute type:`` string

    |  ``ifIPDotted:`` Interface IP.
    |  ``attribute type:`` string

    |  ``ifIPNumeric:`` Interface IP Numeric.
    |  ``attribute type:`` string

    |  ``Network:`` The name of the Network View associated.
    |  ``attribute type:`` string

    |  ``VirtualNetworkMemberName:`` Name of the Local VRF instance.
    |  ``attribute type:`` string

    |  ``VirtualNetworkID:`` The internal identifier for the network.
    |  ``attribute type:`` number

    |  ``VirtualNetworkMemberDescription:`` The description of the VRF as configured on this device.
    |  ``attribute type:`` string

    |  ``RouteDistinguisher:`` Route Distinguisher.
    |  ``attribute type:`` string

    """

    properties = ("id",
                  "DeviceID",
                  "DeviceName",
                  "DeviceIPDotted",
                  "DeviceIPNumeric",
                  "DeviceNetwork",
                  "DeviceVirtualNetworkID",
                  "ifIndex",
                  "ifName",
                  "ifNameSort",
                  "ifIPDotted",
                  "ifIPNumeric",
                  "Network",
                  "VirtualNetworkMemberName",
                  "VirtualNetworkID",
                  "VirtualNetworkMemberDescription",
                  "RouteDistinguisher",
                  )



from ..remote import RemoteModel


class AccessSearchRemote(RemoteModel):
    """
    Active and recently completed access search requests.


    |  ``id:`` The internal natural identifier for this search operation.
    |  ``attribute type:`` number

    |  ``user_name:`` The name of owner of this search.
    |  ``attribute type:`` string

    |  ``search_set_key:`` The internal random generated key that identify this search operation.
    |  ``attribute type:`` string

    |  ``status:`` The current status of the search operation. A value within : 'Starting', 'Running', 'Cancelled', 'Done', 'Error'.
    |  ``attribute type:`` string

    |  ``rules_json:`` The rules definition of the multi-search criteria in a json format.
    |  ``attribute type:`` string

    |  ``created_at:`` The date and time the record was initially created in NetMRI.
    |  ``attribute type:`` datetime

    |  ``updated_at:`` The date and time the record was last modified in NetMRI.
    |  ``attribute type:`` datetime

    |  ``errors:`` The last error that occurred in this search operation.
    |  ``attribute type:`` string

    |  ``general_errors:`` The list or errors occurred during the search operation.
    |  ``attribute type:`` string

    |  ``summary:`` The summary information for this provisioning operation.
    |  ``attribute type:`` string

    """

    properties = ("id",
                  "user_name",
                  "search_set_key",
                  "status",
                  "rules_json",
                  "created_at",
                  "updated_at",
                  "errors",
                  "general_errors",
                  "summary",
                  )



from ..remote import RemoteModel


class AdvSettingOptionRemote(RemoteModel):
    """
    This table list out the advance setting options.


    |  ``id:`` The internal NetMRI identifier for this setting option.
    |  ``attribute type:`` number

    |  ``label:`` The label for this setting option.
    |  ``attribute type:`` string

    |  ``value:`` The value for this setting option.
    |  ``attribute type:`` string

    |  ``adv_setting_def_id:`` The internal NetMRI identifier for setting definition.
    |  ``attribute type:`` number

    """

    properties = ("id",
                  "label",
                  "value",
                  "adv_setting_def_id",
                  )



from ..broker import Broker


class CustomFieldsBroker(Broker):
    controller = "custom_fields"

    def list(self, **kwargs):
        """Returns list of custom fields defined for requested model or list of custom fields defined for model object if id parameter is passed.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param model: Name of the model to work with. Valid values are Device, Interface, JobSpecification, DetectedChange, DevicePhysical, IssueDesc.
             :type model: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param id: Id of the object to get custom fields for.
             :type id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param fields: Array of attributes to use while searching.
             :type fields: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param query: Search pattern.
             :type query: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param allow_blank: Flag which defines whether to return blank custom fields for model object or not.
             :type allow_blank: Boolean

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param limit: Limit the number of records returned.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param start: Record to begin return with.
             :type start: Integer

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("list"), kwargs)

    def list_undefined(self, **kwargs):
        """Returns list of undefined custom field for specified object.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param model: Name of the model to work with. Valid values are Device, Interface, JobSpecification, DetectedChange, DevicePhysical, IssueDesc.
             :type model: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: Object id.
             :type id: Integer

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("list_undefined"), kwargs)

    def import_data(self, **kwargs):
        """Allows importing custom field data using CSV format.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param model: Name of the model to work with. Valid values are Device, Interface, JobSpecification, DetectedChange, DevicePhysical, IssueDesc.
             :type model: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param file_name: The contents of the CSV file with the custom field data to be imported.
             :type file_name: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("import"), kwargs)

    def create_field(self, **kwargs):
        """Allows creating new custom field for specified model.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param model: Name of the model to work with. Valid values are Device, Interface, JobSpecification, DetectedChange, DevicePhysical, IssueDesc.
             :type model: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param name: Name of new custom field.
             :type name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param type: Type of new custom field (integer, datetime, date, string).
             :type type: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("create_field"), kwargs)

    def update_field(self, **kwargs):
        """Allows updating properties of custom field assosiated with specified model (changing name, type).

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param model: Name of the model to work with. Valid values are Device, Interface, JobSpecification, DetectedChange, DevicePhysical, IssueDesc.
             :type model: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param name: Old custom field name.
             :type name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param new_name: New name for custom field.
             :type new_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param type: Old or new custom field type (integer, datetime, date, string).
             :type type: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("update_field"), kwargs)

    def delete_field(self, **kwargs):
        """Allows deleting custom field assosiated with specified model.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param model: Name of the model to work with. Valid values are Device, Interface, JobSpecification, DetectedChange, DevicePhysical, IssueDesc.
             :type model: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param name: Name of custom field to delete.
             :type name: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("delete_field"), kwargs)



from ..broker import Broker


class AuthGroupBroker(Broker):
    controller = "auth_groups"

    def show(self, **kwargs):
        """Shows the details for the specified auth group.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The id of the authentication remote group
             :type id: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return auth_group: The auth group identified by the specified id.
             :rtype auth_group: AuthGroup

            """

        return self.api_request(self._get_method_fullname("show"), kwargs)

    def index(self, **kwargs):
        """Lists the available auth groups. Any of the inputs listed may be be used to narrow the list; other inputs will be ignored. Of the various ways to query lists, using this method is most efficient.

            **Inputs**

            |  ``api version min:`` 3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param id: The id of the authentication remote group
             :type id: Array of Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` id

             :param sort: The data field(s) to use for sorting the output. Default is id. Valid values are id, group_name, auth_service_id, enabled_ind, description, created_at, updated_at, last_logged_time.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each AuthGroup. Valid values are id, group_name, auth_service_id, enabled_ind, description, created_at, updated_at, last_logged_time. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return auth_groups: An array of the AuthGroup objects that match the specified input criteria.
             :rtype auth_groups: Array of AuthGroup

            """

        return self.api_list_request(self._get_method_fullname("index"), kwargs)

    def search(self, **kwargs):
        """Lists the available auth groups matching the input criteria. This method provides a more flexible search interface than the index method, but searching using this method is more demanding on the system and will not perform to the same level as the index method. The input fields listed below will be used as in the index method, to filter the result, along with the optional query string and XML filter described below.

            **Inputs**

            |  ``api version min:`` 3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param auth_service_id: The id of the authentication service, where this group is defined.
             :type auth_service_id: Array of Integer

            |  ``api version min:`` 3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param created_at: The date and time the record was initially created in NetMRI.
             :type created_at: Array of DateTime

            |  ``api version min:`` 3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param description: Description/comment about this remote group.
             :type description: Array of String

            |  ``api version min:`` 3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param enabled_ind: A flag indicating whether this group mappings to NetMRI roles is enabled or disabled.
             :type enabled_ind: Array of Boolean

            |  ``api version min:`` 3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param group_name: The name of the authentication remote group
             :type group_name: Array of String

            |  ``api version min:`` 3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param id: The id of the authentication remote group
             :type id: Array of Integer

            |  ``api version min:`` 3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param last_logged_time: The last connection date and time of a user belonging to this AuthGroup.
             :type last_logged_time: Array of DateTime

            |  ``api version min:`` 3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param updated_at: The date and time the record was last modified in NetMRI.
             :type updated_at: Array of DateTime

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` id

             :param sort: The data field(s) to use for sorting the output. Default is id. Valid values are id, group_name, auth_service_id, enabled_ind, description, created_at, updated_at, last_logged_time.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each AuthGroup. Valid values are id, group_name, auth_service_id, enabled_ind, description, created_at, updated_at, last_logged_time. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param query: This value will be matched against auth groups, looking to see if one or more of the listed attributes contain the passed value. You may also surround the value with '/' and '/' to perform a regular expression search rather than a containment operation. Any record that matches will be returned. The attributes searched are: auth_service_id, created_at, description, enabled_ind, group_name, id, last_logged_time, updated_at.
             :type query: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param xml_filter: A SetFilter XML structure to further refine the search. The SetFilter will be applied AFTER any search query or field values, but before any limit options. The limit and pagination will be enforced after the filter. Remind that this kind of filter may be costly and inefficient if not associated with a database filtering.
             :type xml_filter: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return auth_groups: An array of the AuthGroup objects that match the specified input criteria.
             :rtype auth_groups: Array of AuthGroup

            """

        return self.api_list_request(self._get_method_fullname("search"), kwargs)

    def find(self, **kwargs):
        """Lists the available auth groups matching the input specification. This provides the most flexible search specification of all the query mechanisms, enabling searching using comparison operations other than equality. However, it is more complex to use and will not perform as efficiently as the index or search methods. In the input descriptions below, 'field names' refers to the following fields: auth_service_id, created_at, description, enabled_ind, group_name, id, last_logged_time, updated_at.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_auth_service_id: The operator to apply to the field auth_service_id. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. auth_service_id: The id of the authentication service, where this group is defined. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_auth_service_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_auth_service_id: If op_auth_service_id is specified, the field named in this input will be compared to the value in auth_service_id using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_auth_service_id must be specified if op_auth_service_id is specified.
             :type val_f_auth_service_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_auth_service_id: If op_auth_service_id is specified, this value will be compared to the value in auth_service_id using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_auth_service_id must be specified if op_auth_service_id is specified.
             :type val_c_auth_service_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_created_at: The operator to apply to the field created_at. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. created_at: The date and time the record was initially created in NetMRI. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_created_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_created_at: If op_created_at is specified, the field named in this input will be compared to the value in created_at using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_created_at must be specified if op_created_at is specified.
             :type val_f_created_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_created_at: If op_created_at is specified, this value will be compared to the value in created_at using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_created_at must be specified if op_created_at is specified.
             :type val_c_created_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_description: The operator to apply to the field description. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. description: Description/comment about this remote group. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_description: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_description: If op_description is specified, the field named in this input will be compared to the value in description using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_description must be specified if op_description is specified.
             :type val_f_description: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_description: If op_description is specified, this value will be compared to the value in description using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_description must be specified if op_description is specified.
             :type val_c_description: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_enabled_ind: The operator to apply to the field enabled_ind. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. enabled_ind: A flag indicating whether this group mappings to NetMRI roles is enabled or disabled. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_enabled_ind: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_enabled_ind: If op_enabled_ind is specified, the field named in this input will be compared to the value in enabled_ind using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_enabled_ind must be specified if op_enabled_ind is specified.
             :type val_f_enabled_ind: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_enabled_ind: If op_enabled_ind is specified, this value will be compared to the value in enabled_ind using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_enabled_ind must be specified if op_enabled_ind is specified.
             :type val_c_enabled_ind: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_group_name: The operator to apply to the field group_name. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. group_name: The name of the authentication remote group For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_group_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_group_name: If op_group_name is specified, the field named in this input will be compared to the value in group_name using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_group_name must be specified if op_group_name is specified.
             :type val_f_group_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_group_name: If op_group_name is specified, this value will be compared to the value in group_name using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_group_name must be specified if op_group_name is specified.
             :type val_c_group_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_id: The operator to apply to the field id. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. id: The id of the authentication remote group For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_id: If op_id is specified, the field named in this input will be compared to the value in id using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_id must be specified if op_id is specified.
             :type val_f_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_id: If op_id is specified, this value will be compared to the value in id using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_id must be specified if op_id is specified.
             :type val_c_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_last_logged_time: The operator to apply to the field last_logged_time. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. last_logged_time: The last connection date and time of a user belonging to this AuthGroup. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_last_logged_time: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_last_logged_time: If op_last_logged_time is specified, the field named in this input will be compared to the value in last_logged_time using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_last_logged_time must be specified if op_last_logged_time is specified.
             :type val_f_last_logged_time: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_last_logged_time: If op_last_logged_time is specified, this value will be compared to the value in last_logged_time using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_last_logged_time must be specified if op_last_logged_time is specified.
             :type val_c_last_logged_time: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_updated_at: The operator to apply to the field updated_at. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. updated_at: The date and time the record was last modified in NetMRI. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_updated_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_updated_at: If op_updated_at is specified, the field named in this input will be compared to the value in updated_at using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_updated_at must be specified if op_updated_at is specified.
             :type val_f_updated_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_updated_at: If op_updated_at is specified, this value will be compared to the value in updated_at using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_updated_at must be specified if op_updated_at is specified.
             :type val_c_updated_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` id

             :param sort: The data field(s) to use for sorting the output. Default is id. Valid values are id, group_name, auth_service_id, enabled_ind, description, created_at, updated_at, last_logged_time.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each AuthGroup. Valid values are id, group_name, auth_service_id, enabled_ind, description, created_at, updated_at, last_logged_time. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param xml_filter: A SetFilter XML structure to further refine the search. The SetFilter will be applied AFTER any search query or field values, but before any limit options. The limit and pagination will be enforced after the filter. Remind that this kind of filter may be costly and inefficient if not associated with a database filtering.
             :type xml_filter: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return auth_groups: An array of the AuthGroup objects that match the specified input criteria.
             :rtype auth_groups: Array of AuthGroup

            """

        return self.api_list_request(self._get_method_fullname("find"), kwargs)

    def create(self, **kwargs):
        """Creates a new auth group.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param auth_service_id: The id of the authentication service, where this group is defined.
             :type auth_service_id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param description: Description/comment about this remote group.
             :type description: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` True

             :param enabled_ind: A flag indicating whether this group mappings to NetMRI roles is enabled or disabled.
             :type enabled_ind: Boolean

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param group_name: The name of the authentication remote group
             :type group_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param last_logged_time: The last connection date and time of a user belonging to this AuthGroup.
             :type last_logged_time: DateTime

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return id: The id of the newly created auth group.
             :rtype id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return model: The class name of the newly created auth group.
             :rtype model: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return uri: A URI that may be used to retrieve the newly created auth group.
             :rtype uri: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return auth_group: The newly created auth group.
             :rtype auth_group: AuthGroup

            """

        return self.api_request(self._get_method_fullname("create"), kwargs)

    def update(self, **kwargs):
        """Updates an existing auth group.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The id of the authentication remote group
             :type id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param auth_service_id: The id of the authentication service, where this group is defined. If omitted, this field will not be updated.
             :type auth_service_id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param description: Description/comment about this remote group. If omitted, this field will not be updated.
             :type description: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param enabled_ind: A flag indicating whether this group mappings to NetMRI roles is enabled or disabled. If omitted, this field will not be updated.
             :type enabled_ind: Boolean

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param group_name: The name of the authentication remote group If omitted, this field will not be updated.
             :type group_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param last_logged_time: The last connection date and time of a user belonging to this AuthGroup. If omitted, this field will not be updated.
             :type last_logged_time: DateTime

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return id: The id of the updated auth group.
             :rtype id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return model: The class name of the updated auth group.
             :rtype model: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return uri: A URI that may be used to retrieve the updated auth group.
             :rtype uri: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return auth_group: The updated auth group.
             :rtype auth_group: AuthGroup

            """

        return self.api_request(self._get_method_fullname("update"), kwargs)

    def destroy(self, **kwargs):
        """Deletes the specified auth group from NetMRI.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The id of the authentication remote group
             :type id: Integer

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("destroy"), kwargs)

    def duplicate(self, **kwargs):
        """Duplicate a remote group.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The internal NetMRI identifier for the remote auth group.
             :type id: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return group_id: The new group id.
             :rtype group_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return group_name: The new group name.
             :rtype group_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return group_description: The new group description.
             :rtype group_description: String

            """

        return self.api_request(self._get_method_fullname("duplicate"), kwargs)

    def auth_roles(self, **kwargs):
        """Returns the roles associated with the remote group, and the device groups to which they apply.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The internal NetMRI identifier for the remote auth group.
             :type id: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return auth_roles: The roles assigned to the group users, along with an extra 'device_group_ids' attribute, listing the device groups for which the role applies.
             :rtype auth_roles: Array of AuthRole

            """

        return self.api_request(self._get_method_fullname("auth_roles"), kwargs)



from ..remote import RemoteModel
from infoblox_netmri.utils.utils import check_api_availability


class VrfRouteTargetRemote(RemoteModel):
    """
    The import and export route targets for a Virtual Network Member.


    |  ``VrfRouteTargetStartTime:`` The starting effective time of this record.
    |  ``attribute type:`` datetime

    |  ``VrfRouteTargetEndTime:`` The ending effective time of this record, or empty if still in effect.
    |  ``attribute type:`` datetime

    |  ``VrfRouteTargetChangedCols:`` The fields that changed between this revision of the record and the previous revision.
    |  ``attribute type:`` string

    |  ``VrfRouteTargetTimestamp:`` The date and time this record was collected or calculated.
    |  ``attribute type:`` datetime

    |  ``VrfRouteTargetFirstTime:`` The first time this data element was seen on the network.
    |  ``attribute type:`` datetime


    |  ``DataSourceID:`` The internal NetMRI identifier for the collector NetMRI that collected this data record.
    |  ``attribute type:`` number

    |  ``DeviceID:`` The internal NetMRI identifier for the device from which this data was collected.
    |  ``attribute type:`` number


    |  ``VrfDirection:`` The direction of the RT (import or export).
    |  ``attribute type:`` string

    |  ``VrfMemberID:`` The NetMRI internal identifier for the VRF membership record to which this route target applies.
    |  ``attribute type:`` number

    |  ``VirtualNetworkMemberID:`` The internal NetMRI identifier for the Virtual Network Member that corresponds to this VRF route target.
    |  ``attribute type:`` number

    |  ``VrfRouteTargetID:`` The internal NetMRI identifier for this VRF route target.
    |  ``attribute type:`` number

    |  ``RTType:`` The style of the route target, asn or ipv4.
    |  ``attribute type:`` string

    |  ``RTLeftSide:`` The left-hand portion of the route target; use Type to identify if it is an AS number or and IPv4 address.
    |  ``attribute type:`` number

    |  ``RTRightSide:`` The right-hand portion of the route target.
    |  ``attribute type:`` number

    """

    properties = ("VrfRouteTargetStartTime",
                  "VrfRouteTargetEndTime",
                  "VrfRouteTargetChangedCols",
                  "VrfRouteTargetTimestamp",
                  "VrfRouteTargetFirstTime",
                  "DataSourceID",
                  "DeviceID",
                  "VrfDirection",
                  "VrfMemberID",
                  "VirtualNetworkMemberID",
                  "VrfRouteTargetID",
                  "RTType",
                  "RTLeftSide",
                  "RTRightSide",
                  )

    @property
    @check_api_availability
    def data_source(self):
        """
        The internal NetMRI identifier for the collector NetMRI that collected this data record.
        ``attribute type:`` model
        """
        return self.broker.data_source(**{"VrfRouteTargetID": self.VrfRouteTargetID})

    @property
    @check_api_availability
    def device(self):
        """
        The device from which this VRF route target configuration was collected.
        ``attribute type:`` model
        """
        return self.broker.device(**{"VrfRouteTargetID": self.VrfRouteTargetID})
